'use strict';

/**
 * Module that provides an API to work with nodes.
 * @author Roman Yakimchuk
 * @module
 * @constructor
 * @returns {Object} Instance of class.
 */
function Nodes () {

  /**
   * The names of node variations, known for this class.
   * @author Roman Yakimchuk
   * @private
   * @type {Array} Array of modules names of node variations.
   */
  this.names = [
    'MultilineComment',
    'InlineComment',
    'TheVariableStatement',
    'SimpleAssigmentOperator',
    'CompoundMultipleAssigmentOperator',
    'CompoundDevideAssigmentOperator',
    'CompoundRemaindAssigmentOperator',
    'CompoundPlusAssigmentOperator',
    'CompoundMinusAssigmentOperator',
    'CompoundLeftShiftAssigmentOperator',
    'CompoundSignedRightShiftAssigmentOperator',
    'CompoundUnsignedRightShiftAssigmentOperator',
    'CompoundBitwiseAndAssigmentOperator',
    'CompoundBitwiseXorAssigmentOperator',
    'CompoundBitwiseOrAssigmentOperator',
    'BinaryLogicalOperatorOR',
    'BinaryLogicalOperatorAND',
    'BitwiseNotOperator',
    'BitwiseAndOperator',
    'BitwiseXorOperator',
    'BitwiseOrOperator',
    'LogicalNotOperator',
    'UnaryPlusOperator',
    'UnaryMinusOperator',
    'MultiplicativeMultipleOperator',
    'MultiplicativeDevideOperator',
    'MultiplicativeRemainderOperator',
    'TheLeftShiftOperator',
    'TheSignedRightShiftOperator',
    'TheUnsignedRightShiftOperator',
    'TheLessThanOperator',
    'TheGreaterThanOperator',
    'TheLessThanOrEqualOperator',
    'TheGreaterThanOrEqualOperator',
    'TheEqualsOperator',
    'TheDoesNotEqualsOperator',
    'TheStrictEqualsOperator',
    'TheStrictDoesNotEqualOperator',
    'IncrementOperator',
    'DecrementOperator',
    'SemicolonOperator',
    'TheInOperator',
    'TheInstanceofOperator',
    'TheNewOperator',
    'TheDeleteOperator',
    'TheVoidOperator',
    'TheTypeofOperator',
    'LeftBrace',
    'RightBrace',
    'LeftBracket',
    'RightBracket',
    'LeftParenthesis',
    'RightParenthesis',
    'TheFunctionStatement',
    'TheTryStatement',
    'TheCatchStatement',
    'TheFinallyStatement',
    'TheDoStatement',
    'TheWhileStatement',
    'TheForStatement',
    'TheCaseStatement',
    'TheContinueStatement',
    'TheBreakStatement',
    'TheReturnStatement',
    'TheWithStatement',
    'TheSwitchStatement',
    'TheThrowStatement',
    'TheDebuggerStatement',
    'ColonOperator',
    'ConditionalOperatorQuestionMark',
    'CommaOperator',
    'DotOperator',
    'NewOperator',
    'TheIfStatement',
    'TheElseStatement',
    'BooleanLiteral',
    'Word',
    'FloatNumberLiteral',
    'StringLiteral',
    'RegularExpressionLiteral',
    'IntegerNumberLiteral'
  ];

  /**
   * List of node variations (node modules loaded by this module at module initializing).
   * @author Roman Yakimchuk
   * @type {Array} Array of node variations.
   */
  this.variations = [];

  /**
   * Method that returns list of variations known for this module.
   * @author Roman Yakimchuk
   * @public
   * @this {Object} Module 'Nodes'
   * @member Nodes
   * @returns {Array} Array of node variations.
   */
  this.getVariations = function () {
    return this.variations;
  };

  /**
   * Method that returns list names of node modules, known for this module.
   * @author Roman Yakimchuk
   * @private
   * @this {Object} Module 'Nodes'
   * @member Nodes
   * @returns {Array} Array of modules names of node variations.
   */
  this.getNames = function () {
    return this.names;
  };

  /**
   * Method checks that string presents correct node variation.
   * @author Roman Yakimchuk
   * @public
   * @this {Object} Module 'Nodes'
   * @member Nodes
   * @param {String} string String to check.
   * @param {Array} list Array of strings of parsed source code.
   * @returns {Boolean} If string is correct node variation, method returns {true}, else {false}.
   */
  this.isNode = function (string, list) {
    return this.getVariation.apply(this, arguments) !== null;
  };

  /**
   * Method returns last node instance from list of node instances.
   * @author Roman Yakimchuk
   * @public
   * @this {Object} Module 'Nodes'
   * @member Nodes
   * @param {Array} nodes Array of node instances.
   * @returns {Object} Last node instance from list.
   */
  this.getLast = function (nodes) {
    return nodes.slice(-1)[0];
  };

  /**
   * Method checks that list of nodes is invalid.
   * @author Roman Yakimchuk
   * @public
   * @this {Object} Module 'Nodes'
   * @member Nodes
   * @param {Array} nodes Array of node instances.
   * @returns {Boolean}
   */
  this.isInvalid = function (nodes) {

    if (nodes.length === 0) {
      return false;
    }

    return this.getLast(nodes) === null;

  };

  /**
   * Method converts list of node instances into list of nodes in object notation.
   * @author Roman Yakimchuk
   * @public
   * @this {Object} Module 'Nodes'
   * @member Nodes
   * @param {Array} nodes Array of node instances.
   * @returns {Array} Array of nodes in object notation.
   */
  this.convert = function (nodes) {

    var array, length, i, node, object;

    array = [];

    length = nodes.length;

    if (length === 0) {
      return array;
    }

    for (i = length; i--;) {

      node = nodes[i];

      object = {

        name  : node.getName(),
        types : node.getTypes()

      };

      array.unshift(object);

    }

    return array;

  };

  /**
   * Method that returns variation class for string, that presents correct node variation.
   * @author Roman Yakimchuk
   * @public
   * @this {Object} Module 'Nodes'
   * @member Nodes
   * @param {String} string String, that presents correct node variation.
   * @param {Array} list Array of strings of parsed source code.
   * @returns {Function} Class of node variation.
   */
  this.getVariation = function (string, list) {

    var variations, variation, length, i;

    variations = this.getVariations();

    length = variations.length;

    for (i = 0; i < length; i++) {

      variation = variations[i];

      if (variation.is(string, list)) {
        return variation;
      }

    }

    return null;

  };

  /**
   * Method that pushes variation into module variation stack.
   * @author Roman Yakimchuk
   * @private
   * @this {Object} Module 'Nodes'
   * @member Nodes
   * @param {Function} variation Class of variation.
   * @returns {Object} Module 'Nodes'.
   */
  this.unshiftVariation = function (variation) {

    this.getVariations().unshift(variation);

    return this;

  };

  /**
   * Method that fill stack of variations.
   * @author Roman Yakimchuk
   * @private
   * @this {Object} Module 'Nodes'
   * @member Nodes
   * @returns {Object} Module 'Nodes'.
   */
  this.requireVariations = function () {

    var names, name, variation, length, i;

    names = this.getNames();

    length = names.length;

    for (i = length; i--;) {

      name = names[i];

      variation = require('../nodes/' + name);

      this.unshiftVariation(variation);

    }

    return this;

  };

  /**
   * Method for initializing this module.
   * @author Roman Yakimchuk
   * @private
   * @this {Object} Module 'Nodes'
   * @member Nodes
   * @returns {Object} Module 'Nodes'.
   */
  this.init = function () {

    this.requireVariations();

    return this;

  };

  this.init();

  return this;

}

module.exports = new Nodes();
