'use strict';

/**
 * Module that provides an API for parsing of source code.
 * @author Roman Yakimchuk
 * @module
 * @constructor
 * @returns {Object} Instance of class.
 */
function Parser () {

  /**
   * Method that converts string into node instance, or list of node instances, and pushes them into list of instances.
   * @author Roman Yakimchuk
   * @private
   * @this {Object} Module 'Parser'
   * @member Parser
   * @param {String} string String that presents correct node variation.
   * @param {Array} list Link to array with already pushed node instances.
   * @param {Array} strings Array of strings of parsed source code.
   * @param {Number} index Index of string in array of strings.
   * @returns {Boolean} Returns false, if parsing fails, else true.
   */
  this.parseString = function (string, list, strings, index) {

    var Nodes;

    Nodes = module.require('./nodes');

    var Constructor, instance, strings;

    if (__verbose) {
      console.log(
        '        ', 'Parsing of element'.yellow, '"' + string + '"'
      );
    }

    if (!Nodes.isNode(string, list)) {

      if (string.length <= 1) {

        if (__verbose) {
          console.log(
            '            ', 'Cannot detect type of element.'.red
          );
        }

        list.push(
          null
        );

        return false;

      }

      strings = [
        string.substr(0, 1),
        string.substr(1)
      ];

      if (__verbose) {
        console.log(
          '            ', 'Cannot detect type of element, string will be splitted:'.yellow,
            '"' + string + '"' + ' splitted as '.yellow + '"' + strings[0] + '" + "' + strings[1] + '"'
        );
      }

      return this.makeNodes(
        strings,
        list
      );

    }

    Constructor = Nodes.getVariation(string, list);

    instance = new Constructor (string, list, strings, index);

    if (__verbose) {
      console.log(
        '            ', 'Node successfully detected ('.green + '"' + instance.getName() + '"' + ')!'.green
      );
    }

    list.push(
      instance
    );

    return true;

  };

  /**
   * Method that converts array of strings into array of node instances.
   * @author Roman Yakimchuk
   * @private
   * @this {Object} Module 'Parser'
   * @member Parser
   * @param {Array} strings Array of strings of parsed source code.
   * @param {Array} nodes Link to array with already pushed node instances.
   * @returns {Boolean}
   */
  this.makeNodes = function (strings, nodes) {

    var string, length, i;

    length = strings.length;

    for (i = 0; i < length; i++) {

      string = strings[i];

      if (!this.parseString(string, nodes, strings, i)) {
        return false;
      }

    }

    return true;

  };

  /**
   * Method that parse source code, and returns array of instances of nodes.
   * @author Roman Yakimchuk
   * @public
   * @this {Object} Module 'Parser'
   * @member Parser
   * @param {String} source Source code for parsing.
   * @returns {Array} Array of instances of nodes.
   */
  this.parse = function (source) {

    var strings, nodes, length, i;

    if (__verbose) {
      console.log(
        '    ', 'Parsing was started..'.yellow
      );
    }

    source = source.trim();

    strings = source.split(/(\s|;|[\$\_a-zA-Z]{1}[a-zA-Z0-9\$\_]*|[0-9]+\.[0-9]+|[0-9]+|\"[^\"]*\"|\'[^\']*\'|\/\*[\d\D]*?\*\/|\/\/.*\n|\/\/.*$|\/{1}.*[^\/]{3}\/{1})/); // @todo: исправить баг парсинга регулярки в условиях однострочного комментария

    length = strings.length;

    for (i = length; i--;) {

      if (['', ' ', "\n"].indexOf(strings[i]) >= 0) {
        strings.splice(i, 1);
      }

    }

    nodes = [];

    this.makeNodes(
      strings,
      nodes
    );

    return nodes;

  };

}

module.exports = new Parser();
